/**************************************************************
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 *************************************************************/

#ifndef INCLUDED_DRAWINGLAYER_ATTRIBUTE_SDRLINESTARTENDATTRIBUTE_HXX
#define INCLUDED_DRAWINGLAYER_ATTRIBUTE_SDRLINESTARTENDATTRIBUTE_HXX

#include <drawinglayer/drawinglayerdllapi.h>
//////////////////////////////////////////////////////////////////////////////
// predefines

namespace basegfx {
    class B2DPolyPolygon;
}

namespace drawinglayer { namespace attribute {
    class ImpSdrLineStartEndAttribute;
}}

//////////////////////////////////////////////////////////////////////////////

namespace drawinglayer
{
	namespace attribute
	{
		class DRAWINGLAYER_DLLPUBLIC SdrLineStartEndAttribute
		{
        private:
            ImpSdrLineStartEndAttribute*               mpSdrLineStartEndAttribute;

		public:
            /// constructors/assignmentoperator/destructor
			SdrLineStartEndAttribute(
				const basegfx::B2DPolyPolygon& rStartPolyPolygon, 
                const basegfx::B2DPolyPolygon& rEndPolyPolygon,
				double fStartWidth, 
                double fEndWidth, 
                bool bStartActive, 
                bool bEndActive, 
                bool bStartCentered, 
                bool bEndCentered);
			SdrLineStartEndAttribute();
			SdrLineStartEndAttribute(const SdrLineStartEndAttribute& rCandidate);
			SdrLineStartEndAttribute& operator=(const SdrLineStartEndAttribute& rCandidate);
			~SdrLineStartEndAttribute();

            // checks if the incarnation is default constructed
            bool isDefault() const;

            // compare operator
			bool operator==(const SdrLineStartEndAttribute& rCandidate) const;

			// data read access
			const basegfx::B2DPolyPolygon& getStartPolyPolygon() const;
			const basegfx::B2DPolyPolygon& getEndPolyPolygon() const;
			double getStartWidth() const;
			double getEndWidth() const;
			bool isStartActive() const;
			bool isEndActive() const;
			bool isStartCentered() const;
			bool isEndCentered() const;
		};
	} // end of namespace attribute
} // end of namespace drawinglayer

//////////////////////////////////////////////////////////////////////////////

#endif //INCLUDED_DRAWINGLAYER_ATTRIBUTE_SDRLINESTARTENDATTRIBUTE_HXX

// eof
